/**
* @file nonvolatile.h
* @brief This file is an interface for nonvolatile memory
* @addtogroup NONVOLATILE nonvolatile
* @ingroup DRIVER
* @{
* @author Ryan Tensmeyer, Levi Balling(9/23/2014)
* @version 1
* @date 2/11/2014
* @details Used to read to and write from nonvolatile memory
**/

#ifndef NONVOLATILE_H
#define NONVOLATILE_H

/**
* INCLUDES
**/
#include "../kernel/constants.h"
#include "../kernel/taskCntrl.h"


/**
* DEFINES
**/
#define MAX_NV_RAM_BUFFER	1024

///**


typedef enum
{
	NV_NONE = 0,
	NV_DIRTY_BYTE = 1, ///this value is determined by the system and written to NV to determine that it's unsafe to continue
	NV_BLE_NAME = 2, ///Read/Write,not Required, the name to be displayed over ble
	NV_MACHINE_NAME = 3, ///read,write,not required. The name of the machine
	NV_USER_NAME = 4, ///read, write, not required. The name of the user
	NV_UNIQUE_ID = 5, ///read, wriet, not required. The unique ID for error reporting over ifit
	NV_WORKOUT_VARS = 6, ///read, wriet, not required. IDK
	NV_RESERVED = 7, ///reserved
	NV_CONFIG_FILE = 8, ///read, (protected write) required. writing to config file is only valid if it's empty
	NV_LOG_FILE = 9, ///read, write, required
	NV_LOG_PAGE_WRITES = 10, ///readonly, required
	NV_FILE_PAGE_WRITES = 11 ///readonly, required

} MACHINE_CONFIG_ID;

typedef enum
{
	DIRTY_BYTE_VALUE = 0xFE,	///1111 1110 we write 0s, not ones
	CLEAN_BYTE_VALUE = 0x7E,   ///0111 1110
	FIRST_UPDATE_VALUE	= 0x00, ///there is nothing there
	ERASED_UPDATE_VALUE = 0xFF ///the memory has been erased
} DIRTY_BYTE_VALUES;


/**
* UNIONS, STRUCTS, CLASSES
**/

typedef struct nv_page_header
{
	uint8_t	pageId;
	uint8_t reserved;
	uint16_t flashPageWrittenAmount;
	uint8_t  reservedPadding[12]; ///we want 16 bytes of data for the file header
} NV_PAGE_HEADER;

typedef struct nonvolatile_memory_chunk
{
	uint8_t memory_chunk_id; //MACHINE_CONFIG_ID
	uint8_t			memory_chunk_size; ///this is how big a single memory is(2 or 4 bytes, etc.)
	uint8_t			current_memory_chunk_counter; ///this will be used to determine how far into the memory chunk we should read.
	uint16_t		current_memory_pointer_offset; ///this is the offset from the start location to
	///the current value location.(essentially current_memory_chunk_counter * memory_chunk_size
	uint8_t			memory_chunk_max_count; ///how many chunks to store here
	const uint8_t	*memory_section_start_location; ///the pointer to the start of the array of data
	///example usage
	///will return the pointer to the latest value
	///&nv->memory_sections[i].memory_section_start_location[
	///(nv->memory_sections[i].current_memory_chunk_counter * nv->memory_sections[i].memory_chunk_size)]
} NONVOLATILE_MEMORY_SECTION;

typedef struct log_data_page_obj
{
	uint16_t			dataBlockSize; ///this is how large the data we will be storing is.
	uint8_t			dataSectionCount; ///this is how many logging data sections we have
	uint8_t			currentDataBlock; ///this is the array offset for the current data block
	uint8_t			currentDirtyByte; ///this is the array offset for the current dirty byte
	uint16_t		dirtySectionCount; ///this is how many dirty bytes we have
	BOOL			dirtyByteReturn; ///this is the value that is returned from a dirty byte ID request.
	const uint8_t	*dataStart;
	const uint8_t	*dirtyByteStart;
} LOG_DATA_PAGE_OBJ;

typedef struct nonvolatile_obj
{
	uint8_t			memory_section_count;
	BOOL			busyBit;
	LOG_DATA_PAGE_OBJ	logDataPageObj;
	///pointers at end of structs
	uint8_t			NV_RAM_BUFFER[MAX_NV_RAM_BUFFER];
	NONVOLATILE_MEMORY_SECTION *memory_sections;
	const uint8_t		*configFile;//pointer to the configuration file
	const uint8_t		*currentFilePageStart;
	const uint8_t		*blankFilePageStart;
	const uint8_t		*uncleanPageStart;
	const uint8_t		*currentLogPageStart;
	TASK *task;
} NONVOLATILE_OBJ;


/**
* FUNCTIONS
**/

/**
* @fn initNonvolatile
* \brief Initializes the nonvolatile memory
 * @param nonvolatileTask the task that will be called when a change occurs
**/
void initNonvolatile(TASK *nonvolatileTask, const uint8_t *configAddress);

/**
 * @fn isNVBusy
 * @return determines if the NV is in a safe state to be read from or used
 * @retval TRUE - NV is unsafe
 * @retval FALSE - nv is safe to use
 */
BOOL isNVBusy();

/**
 * @fn nonvolatile_callback
 * @brief this function is meant to be a callback that handles an asynchronous NV operation
 * @param sys_evt Defined by each system, SUCCESS, or FAILURE
 */
void nonvolatile_callback(uint32_t sys_evt);
/***********************************
** GETTERS
***********************************/

/**
 * @fn getNVMemorySection
 * @param memory_section_id - is the id for which memory section to recover
 * @return pointer to the data in nv memory
 */
const uint8_t *getNVMemorySection(MACHINE_CONFIG_ID memory_section_id);


/***********************************
** SETTERS
***********************************/
/**
 * @fn writeNVMemorySection
 * @param memory_section_id - the section of memory to write to
 * @param data a pointer to the data to write
 * @param offset - how far into the section to write, may not be supported
 * @param size - how many bytes to write, may not be supported
 * @return TRUE if nv operation successful or in queue. FALSE if ID not supported, or other cases
 */
BOOL writeNVMemorySection(MACHINE_CONFIG_ID memory_section_id, uint8_t *data, uint16_t offset,
						  uint16_t size);



#endif /** NONVOLATILE_H **/

/**! @} **/
